<?php

namespace SGalinski\Tinymce4Rte\Editors;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (http://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Backend\Form\FormEngine;
use TYPO3\CMS\Backend\Rte\AbstractRte;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * A RTE using TinyMCE
 */
class RteBase extends AbstractRte {
	/**
	 * Indicates that the tinymce is loaded
	 *
	 * @var bool
	 */
	static protected $coreLoaded = FALSE;

	/**
	 * Draws the RTE as a form field or whatever is needed (inserts JavaApplet, creates iframe, renders ....)
	 * Default is to output the transformed content in a plain textarea field. This mode is great for debugging transformations!
	 *
	 * @param FormEngine $parentObject Reference to parent object, which is an instance of the TCEforms.
	 * @param string $table The table name
	 * @param string $field The field name
	 * @param array $row The current row from which field is being rendered
	 * @param array $PA Array of standard content for rendering form fields from TCEforms. See TCEforms for details on this. Includes for instance the value and the form field name, java script actions and more.
	 * @param array $specConf "special" configuration - what is found at position 4 in the types configuration of a field from record, parsed into an array.
	 * @param array $thisConfig Configuration for RTEs; A mix between TSconfig and otherwise. Contains configuration for display, which buttons are enabled, additional transformation information etc.
	 * @param string $RTEtypeVal Record "type" field value.
	 * @param string $RTErelPath Relative path for images/links in RTE; this is used when the RTE edits content from static files where the path of such media has to be transformed forth and back!
	 * @param integer $thePidValue PID value of record (true parent page id)
	 * @return string HTML code for RTE!
	 */
	public function drawRTE(
		FormEngine $parentObject, $table, $field, $row, $PA, $specConf,
		$thisConfig, $RTEtypeVal, $RTErelPath, $thePidValue
	) {
		// render the tinymce textarea
		$value = $this->transformContent(
			'rte', $PA['itemFormElValue'], $table, $field, $row, $specConf, $thisConfig, $RTErelPath, $thePidValue
		);

		// render RTE field
		$editorId = uniqid();
		$code = $this->triggerField($PA['itemFormElName']);
		$code .= '<textarea id="editor' . $editorId . '" class="tinymce4_rte"
			name="' . htmlspecialchars($PA['itemFormElName']) . '"
			rows="20" cols="100">' . GeneralUtility::formatForTextarea($value) . '</textarea>';

		// add dedicated RTE configuration
		$elementParts = preg_replace('/^(TSFE_EDIT\\[data\\]\\[|data\\[)/', '', $PA['itemFormElName']);
		$elementParts = preg_replace('/\\]$/', '', $elementParts);
		$elementParts = explode('][', $elementParts);
		list($typoscriptConfigurationPid, $pid) = BackendUtility::getTSCpid(
			trim($elementParts[0]), trim($elementParts[1]), $thePidValue
		);
		$rteConfiguration = rawurlencode(
			$this->getRteConfiguration($specConf, $RTEtypeVal, $pid, $typoscriptConfigurationPid, $elementParts)
		);

		$languageId = max($row['sys_language_uid'], 0);
		$language = ($GLOBALS['LANG']->lang === '' ? 'default' : $GLOBALS['LANG']->lang);
		$code .= '
			<script type = "text/javascript" >
				var RTE = RTE || {};
				RTE["editor' . $editorId . '"] = {};
				RTE["editor' . $editorId . '"].rteConfiguration = "' . $rteConfiguration . '";
				RTE["editor' . $editorId . '"].typo3ContentLanguage = "' . $language . '";
				RTE["editor' . $editorId . '"].sys_language_content = parseInt(' . $languageId . ');
			</script>';

		// load the tinymce code and it's configuration
		if (!self::$coreLoaded) {
			self::$coreLoaded = TRUE;
			$userOrPageProperties = BackendUtility::getModTSconfig($thePidValue, 'RTE');

			/** @var \tinyMCE $tinyMCE */
			require_once(ExtensionManagementUtility::extPath('tinymce') . 'class.tinymce.php');
			$tinyMCE = GeneralUtility::makeInstance('tinyMCE');
			$tinyMCE->loadConfiguration($userOrPageProperties['properties']['default.']['tinymceConfiguration']);
			if ($userOrPageProperties['properties']['default.']['contentCSS'] !== '') {
				$tinyMCE->addConfigurationOption(
					'content_css',
					'/' . $userOrPageProperties['properties']['default.']['contentCSS']
				);
			}
			$code .= $tinyMCE->getJS();

			$imageModule = BackendUtility::getModuleUrl('rtehtmlarea_wizard_select_image');
			$linkModule = BackendUtility::getModuleUrl('rtehtmlarea_wizard_element_browser');
			$code .= '
			<script type = "text/javascript" >
				var RTE = RTE || {};
				RTE.linkToImageModule = ' . GeneralUtility::quoteJSvalue($imageModule) . ';
				RTE.linkToLinkModule = ' . GeneralUtility::quoteJSvalue($linkModule) . ';
			</script>';
		}

		return $code;
	}

	/**
	 * @param array $configuration
	 * @param string $typeValue
	 * @param int $pid
	 * @param int $typoscriptConfigurationPid
	 * @param array $elementParts
	 * @return string
	 */
	public function getRteConfiguration(
		array $configuration, $typeValue, $pid, $typoscriptConfigurationPid, array $elementParts
	) {
		$parameters = BackendUtility::getSpecConfParametersFromArray($configuration['rte_transform']['parameters']);
		return $elementParts[0] . ':' . $elementParts[1] . ':' . $elementParts[2] . ':' . (int) $pid . ':' .
		$typeValue . ':' . (int) $typoscriptConfigurationPid . ':' . $parameters['imgpath'];
	}

}

?>